// Motor.h
#ifndef PLAYER_MIN_KILLABLE
#define PLAYER_MIN_KILLABLE 0
#endif

#ifdef PLAYER_MOGGY_STYLE
// right: 0 + frame
// left: 2 + frame
// up: 4 + frame
// down: 6 + frame
unsigned char *player_frames [] = {
	sprite_1_a, sprite_2_a, sprite_3_a, sprite_4_a,
	sprite_5_a, sprite_6_a, sprite_7_a, sprite_8_a
};
#else
#ifdef PLAYER_BOOTEE
// vy = 0: 0 + facing
// vy < 0: 1 + facing
// vy > 0: 2 + facing
unsigned char *player_frames [] = {
	sprite_5_a, sprite_6_a, sprite_7_a, sprite_8_a,
	sprite_1_a, sprite_2_a, sprite_3_a, sprite_4_a
};
#else
#ifdef PLAYER_ALTERNATE_ANIMATION
// Alternate animation:
// 0 1 2 + facing = walk, 0 = stand, 3 = jump/fall
unsigned char *player_frames [] = {
	sprite_5_a, sprite_6_a, sprite_7_a, sprite_8_a,
	sprite_1_a, sprite_2_a, sprite_3_a, sprite_4_a
};
#else
// Normal animation:
// 0 1 2 3 + facing: walk, 1 = stand. 8 + facing = jump/fall
unsigned char *player_frames [] = {
	sprite_5_a, sprite_6_a, sprite_7_a, sprite_6_a,
	sprite_1_a, sprite_2_a, sprite_3_a, sprite_2_a,
	sprite_8_a, sprite_4_a
};
#endif
#endif
#endif
unsigned char *enem_frames [] = {
	sprite_9_a, sprite_10_a, sprite_11_a, sprite_12_a, 
	sprite_13_a, sprite_14_a, sprite_15_a, sprite_16_a
};

// Funciones:

#ifdef COMPRESSED_LEVELS
void prepare_level (unsigned char level) {
	unpack ((unsigned int) levelset [level].leveldata_c, MAP_DATA);
	unpack ((unsigned int) levelset [level].tileset_c, (unsigned int) (tileset));
	unpack ((unsigned int) levelset [level].spriteset_c, (unsigned int) (sprite_1_a - 16));
	n_pant = levelset [level].ini_pant;
	player.x = levelset [level].ini_x << 10;
	player.y = levelset [level].ini_y << 10;
	n_bolts = *((unsigned char *) (NBOLTS_PEEK));
}
#endif

extern unsigned char playing;
extern unsigned char s_joy_read;

// Engine globals (for speed) & size!
unsigned char gpx, gpy, gpd, gpc, gpt;
unsigned char gpxx, gpyy, gpcx, gpcy;
unsigned char possee, hit_v, hit_h, hit, wall;
unsigned char gpen_x, gpen_y, gpen_cx, gpen_cy, gpen_xx, gpen_yy, gpaux;
unsigned char tocado, active;
unsigned char gpit, gpjt;
unsigned char pastilla_rnd;
unsigned char *map_pointer;
unsigned char malotes_t;
unsigned char malotes_k;
#ifdef PLAYER_CAN_FIRE
unsigned char blx, bly;
#endif

void draw_scr (void);

// utils

void srand (unsigned int new_seed) {
	seed [0] = new_seed;	
}

unsigned char rand (void) {
	unsigned char res;
	
	#asm
		.rand16
			ld	hl, _seed
			ld	a, (hl)
			ld	e, a
			inc	hl
			ld	a, (hl)
			ld	d, a
			
			;; Ahora DE = [SEED]
						
			ld	a,	d
			ld	h,	e
			ld	l,	253
			or	a
			sbc	hl,	de
			sbc	a, 	0
			sbc	hl,	de
			ld	d, 	0
			sbc	a, 	d
			ld	e,	a
			sbc	hl,	de
			jr	nc,	nextrand
			inc	hl
		.nextrand
			ld	d,	h
			ld	e,	l
			ld	hl, _seed
			ld	a,	e
			ld	(hl), a
			inc	hl
			ld	a,	d
			ld	(hl), a
			
			;; Ahora [SEED] = HL
		
			ld 	hl, _asm_int
			ld	a,	e	
			ld	(hl), a
			inc	hl
			ld	a,	d
			ld	(hl), a
			
			;; Ahora [ASM_INT] = HL
	#endasm
	
	res = asm_int [0];

	return res;
}

unsigned int abs (int n) {
	if (n < 0)
		return (unsigned int) (-n);
	else 
		return (unsigned int) n;
}

void init_player (void) {
	// Inicializa player con los valores iniciales
	// (de ah lo de inicializar).
	
#ifndef COMPRESSED_LEVELS
	player.x = 			PLAYER_INI_X << 10;
	player.y = 			PLAYER_INI_Y << 10;
#endif	
	player.vy = 		0;
	player.vx = 		0;
	player.cont_salto = 1;
	player.saltando = 	0;
	player.frame = 		0;
	player.subframe = 	0;
    player.keys = 0;
#ifdef PLAYER_MOGGY_STYLE
	player.facing = FACING_DOWN;
	player.facing_v = player.facing_h = 0xff;
#else
	player.facing = 	1;
#endif	
	player.estado = 	EST_NORMAL;
	player.ct_estado = 	0;
#if !defined(COMPRESSED_LEVELS) || defined(REFILL_ME)	
	player.life = 		PLAYER_LIFE;
#endif
	player.objs =		0;
	player.disparando = 0;
#ifdef MAX_AMMO
#ifdef INITIAL_AMMO
	player.ammo = INITIAL_AMMO
#else
	player.ammo = MAX_AMMO;
#endif
#endif	

	pant_final = SCR_FIN;
	
	pastilla_rnd = (rand() % 8);
	if(pastilla_rnd == SCR_INICIO)
	    ++pastilla_rnd;

#ifdef TIMER_ENABLE
	ctimer.count = 0;
	ctimer.zero = 0;
	ctimer.frames = TIMER_LAPSE;
	ctimer.t = TIMER_INITIAL;
	ctimer.on = 1;
#endif

}

unsigned char collide (unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2) {
	// Colisin segura y guarra.
	unsigned char l1x, l1y, l2x, l2y;
	
	// espera unos segundos antes de estar activo
	if(malotes[0].wait)
	    return 0;
	    
	l1x = (x1 > 4) ? x1 - 4 : 0;
	l2x = x1 + 4;
	l1y = (y1 > 4) ? y1 - 4 : 0;
	l2y = y1 + 4;
	return (x2 >= l1x && x2 <= l2x && y2 >= l1y && y2 <= l2y);
}

	

void step (void) {
	#asm
		ld a, 16
		out (254), a
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		xor 16
		out (254), a
	#endasm	
}

void cortina (void) {
	#asm
		;; Antes que nada vamos a limpiar el PAPER de toda la pantalla
		;; para que no queden artefactos feos
		
		ld	de, 22528			; Apuntamos con DE a la zona de atributos
		ld	b,	3				; Procesamos 3 tercios
	.clearb1
		push bc
		
		ld	b, 255				; Procesamos los 256 atributos de cada tercio
	.clearb2
	
		ld	a, (de)				; Nos traemos un atributo
		and	199					; Le hacemos la mscara 11000111 y dejamos PAPER a 0
		ld	(de), a				; Y lo volvemos a poner
		
		inc de					; Siguiente atributo
	
		djnz clearb2
		
		pop bc
		djnz clearb1
		
		;; Y ahora el cdigo original que escrib para UWOL:	
	
		ld	a,	8
	
	.repitatodo
		ld	c,	a			; Salvamos el contador de "repitatodo" en 'c'
	
		ld	hl, 16384
		ld	a,	12
	
	.bucle
		ld	b,	a			; Salvamos el contador de "bucle" en 'b'
		ld	a,	255
	
	.bucle1
		sla (hl)
		inc hl
		dec a
		jr	nz, bucle1
			
		ld	a,	255
	.bucle2
		srl (hl)
		inc hl
		dec a
		jr	nz, bucle2
			
		ld	a,	b			; Restituimos el contador de "bucle" a 'a'
		dec a
		jr	nz, bucle
	
		ld	a,	c			; Restituimos el contador de "repitatodo" a 'a'
		dec a
		jr	nz, repitatodo
	#endasm
}


void paint_marco (void) {
	unpack ((unsigned int) (s_marco), 16384);
	print_number2 (LIFE_X, LIFE_Y, player.life);

    if(player.keys & F_PASTI)
        draw_coloured_tile(15 ,21 , 26);
    if(player.keys & F_MALETA)
        draw_coloured_tile(17 ,21 , 42);

	if(player.keys & F_CHAMPI)
        draw_coloured_tile(20 ,21 , 23);
    if(player.keys & F_ENDKEY)
        draw_coloured_tile(22 ,21 , 25);

}

// Game

void espera_activa (int espera) {
	do{
        gpit = 0xFA; // 250
		do { 
		    gpjt = 1; // do nothing, but do something ;)
		}while (--gpit);
		if (sp_GetKey ()) break;
	}while (espera--);
}

void espera_forzosa (int espera) {
	gpit = 0xFA; // 250
	do{
		 while(--gpit);
    } while(espera--);
}

void game_credits(void) {
	espera_forzosa (200);

	print_str (1, 19, 70, "GFX, CONCEPT: JACK SHADOW", FALSE);
	print_str (1, 18, 70, "CODE, GFX: D_SKYWALK", FALSE);
	print_str (1, 20, 70, "MUSIC: SERGEEO", FALSE);

	print_str (4, 22, 39, " FAMILIARGAMEJAM ENTRY ", FALSE);
	print_str (7, 23, 39, " DANTOINE STUDIO ", FALSE);
	sp_UpdateNow ();

	#asm
		; Music generated by beepola
		call musicstart3
	#endasm

	#asm
	  di
	#endasm
	
	espera_forzosa (50);
}


void ending_cured (void) {
	blackout ();
	unpack ((unsigned int) (s_ending), 16384);

	print_str (10, 0, 70, "YOU ARE CURED", TRUE);
	sp_UpdateNow ();
    game_credits ();
}


void game_ending (void) {
	gpit = 2;
	do{
		peta_el_beeper (9);
	}while (--gpit);

	blackout ();
	unpack ((unsigned int) (s_ending), 16384);

	sp_UpdateNow ();
    game_credits ();
}

/*
 * COMBAT FUNCTIONS
 */

void combat_show_ptr(unsigned char * ptr){
    print_str(1 + (7 * gpxx), 19 + (2 * gpyy), 22, ptr, FALSE); // paint user pointer
}


void combat_update_stats (void){
    print_number (6, 4, 23, player.c_hp);
    print_number (6, 5, 23, player.c_mad);
    print_number (26, 4, 23, malotes [0].life);
    print_number (26, 5, 23, malotes [0].mad);
    combat_show_ptr(">");
	sp_UpdateNow ();
}


void clean_msg (void){
    hit = 17;
    do{
        print_str (14 + hit, 20, 23, " ", FALSE);
        print_str (14 + hit, 21, 23, " ", FALSE);
    }while(--hit);
}

void enemy_attack (unsigned char enemy) {

    print_str (23, 20, 22, " ATTACKS", FALSE);

    switch(enemy){
        case EN_YOU:
            print_str (16, 20, 22, " MYSELF", FALSE);
            sp_UpdateNow ();
            peta_el_beeper (6);
            if(hit_v){
                if((!gpxx) && (gpyy))
                    hit = rand() % 2 + 1;
                else
        	        hit = rand() % 5;
    	    }else
    	        hit = rand() % 8;
	        break;
        case EN_GUY: 
            print_str (16, 20, 22, "WACKMAN", FALSE);
            sp_UpdateNow ();
            peta_el_beeper (7);
            if(player.keys & F_MALETA)
    	        hit = rand() % 6;
            else{
    	        hit = rand() % 8;
    	        if(!hit_v)
    	            ++hit;
    	    }
	        if(hit){
    	        if(hit_v > hit)
	                hit = 1;
	            else
	                hit -= hit_v;
	        }
	        break;
	    default:
            print_str (16, 20, 22, "NURSADY", FALSE);
            sp_UpdateNow ();
            peta_el_beeper (11);
            if(player.keys & F_PASTI){
	            hit = rand() % 2;
	        }else
	            hit = rand() % 8;
	        break;
    }

    if(hit){
        switch(enemy){
            case EN_YOU:
            case EN_GUY:         
                print_str (17, 21, 23, "HITS ", FALSE);
                print_number (22, 21, 23, hit);
                if(hit >= player.c_hp)
                    player.c_hp = 0;
                else
                    player.c_hp -= hit;
                break;
    	    default:
    	        if(rand()%2) {
                    print_str (17, 21, 23, "HITS ", FALSE);
                    print_number (22, 21, 23, hit);
                    if(hit >= player.c_hp)
                        player.c_hp = 0;
                    else
                        player.c_hp -= hit;
    	        }else {
                    print_str (17, 21, 23, "INJECTS ", FALSE);
                    print_number (25, 21, 23, hit);
                    player.c_mad += hit;
                    if(player.c_mad > 50)
                        player.c_mad = 50;
                }
                break;
        }
        peta_el_beeper (5);
    }else{
        print_str (17, 21, 23, "FAIL    ", FALSE);
    }
    
    sp_UpdateNow ();
    espera_forzosa (200);
    clean_msg();
}


void combat_talk (unsigned char enemy) {
    combat_show_ptr(" ");
    print_str (15, 20, 22, "USE INSANITY AND", FALSE);
    sp_UpdateNow ();
    peta_el_beeper (11);
    espera_forzosa (100);

    switch(enemy){
        case EN_YOU:
        case EN_NUR: 
            if(player.keys & F_PASTI)
                hit = rand() % 5 + 3;
            else
	            hit = rand() % 3;
	        break;
	    default:
	        hit = 0;
	        break;
    }

    if(hit){
        print_str (17, 21, 23, "HURT ", FALSE);
        print_number (22, 21, 23, hit);
        peta_el_beeper (6);
        malotes [0].mad += hit;
        if(malotes [0].mad > 50)
            malotes [0].mad = 50;

    }else{
        print_str (17, 21, 23, "FAIL    ", FALSE);
    }
    
    sp_UpdateNow ();
    espera_forzosa (200);
    clean_msg();
}

void combat_punch (unsigned char enemy) {
    combat_show_ptr(" ");
    print_str (16, 20, 22, "YOU PUNCH AND", FALSE);
    sp_UpdateNow ();
    peta_el_beeper (6);
    espera_forzosa (100);

    switch(enemy){
        case EN_YOU:
	            hit = rand() % 5;
	            if(hit_v && (!hit))
	                hit = 1;
            break;
        case EN_GUY: 
            if(player.keys & F_MALETA)
                hit = rand() % 5 + 4;
            else
	            hit = rand() % 3;
	        if(hit_v)
	            ++hit;
	        break;
	    default:
	        hit = 0;
    }

    if(hit){
        // add charge
        hit += hit_v;
        print_str (17, 21, 23, "HITS ", FALSE);
        print_number (22, 21, 23, hit);
        peta_el_beeper (5);
        if(hit >= malotes [0].life)
            malotes [0].life  = 0;
        else
            malotes [0].life -= hit;
    }else{
        print_str (17, 21, 23, "FAIL    ", FALSE);
    }
    
    sp_UpdateNow ();
    espera_forzosa (200);
    clean_msg();
}


void guard(){
    combat_show_ptr(" ");
    print_str (16, 20, 22, "YOU GUARD AND", FALSE);
    sp_UpdateNow ();
    espera_forzosa (100);
    
    print_str (17, 21, 23, "CHARGE ", FALSE);
    hit_v = 3;
    sp_UpdateNow ();
    espera_forzosa (200);
    clean_msg();
    
}

void combat (unsigned char enemy) {

	cortina ();
    red_screen();
    unpack ((unsigned int) (s_combat), 16384);
    gpit = 5;
    switch(enemy){
        case EN_YOU:
            print_str (22, 2, 23, "YOU", FALSE);
            print_str (21, 4, 23, "HP:", FALSE);
	        malotes [0].life  = 10;
	        malotes [0].mad = 9;
            print_str (21, 5, 23, "MAD:", FALSE);
            break;
        case EN_GUY: // show guy
	        hide_area(11);
            print_str (22, 2, 23, "WACKMAN", FALSE);
            
            print_str (21, 4, 23, "HP:", FALSE);
            malotes [0].life  = 6;
            malotes [0].mad = 73;
            print_str (21, 5, 23, "MAD:", FALSE);
            break;
        case EN_NUR: // show nurse
            hide_area(15);
            print_str (22, 2, 23, "NURSADY", FALSE);
            malotes [0].life  = 25;
            malotes [0].mad = 30;
            print_str (21, 4, 23, "HP:", FALSE);
            print_str (21, 5, 23, "MAD:", FALSE);
    }
    sp_UpdateNow ();
	espera_forzosa (80);
	
	gpxx = 0;
    gpyy = 0;
    hit_v = 0;

	
	// player refreshed and fail state by default
    player.estado = EST_FAIL;
	player.c_mad = 9 + (7 * (player.keys & F_PASTI));
	player.c_hp = 10;
    print_str (2, 2, 23, "ME", FALSE);
    print_str (1, 4, 23, "HP:", FALSE);
    print_str (1, 5, 23, "MAD:", FALSE);

    // menu   
	print_str (1, 19, 22, ">PUNCH  TALK", FALSE);
	print_str (2, 21, 22,  "GUARD  RUN", FALSE);

    combat_update_stats ();

	#asm
		; Music generated by beepola
		call musicstart2
	#endasm

    while(1) {
    
	    #asm
	      di
	    #endasm

	    combat_show_ptr(" ");

	    gpit = (joyfunc) (&keys); // Leemos del teclado

	    if ((gpit & sp_UP) == 0){
	        if(gpyy)
                --gpyy;
        }
        if ((gpit & sp_DOWN) == 0){
            if(!gpyy)
                ++gpyy;
        }
        if ((gpit & sp_RIGHT) == 0){
            if(!gpxx)
                ++gpxx;
        }
          if ((gpit & sp_LEFT) == 0){
            if(gpxx)
                --gpxx;
        }
        
        if ((gpit & sp_FIRE) == 0){
            if(gpxx && gpyy){
                peta_el_beeper (8);
                break;
            }
            
            if((gpxx) && (!gpyy)) {  // TALK OPT
                combat_talk(enemy);
                enemy_attack(enemy);
            } 
            
            if((!gpxx) && (!gpyy)) { // PUNCH OPT
                combat_punch(enemy);
                enemy_attack(enemy);
                hit_v = 0;
            }

            if((!gpxx) && (gpyy)) {  // GUARD OPT
                guard();
                enemy_attack(enemy);
            } 


            // check player stats                
            if((!player.c_hp)||(player.c_mad == 50)){
                combat_update_stats ();
                if(enemy != EN_YOU)
                    print_str (16, 20, 22, "SLEEP AGAIN...", FALSE);
                else
                    print_str (15, 20, 22, "SLEEP FOREVER...", FALSE);
                break;
            }
            
            //check enemy stats
            if((!malotes [0].life)||(malotes [0].mad == 50)){
                combat_update_stats ();

                player.estado = EST_BEAT;
                malotes_k |= enemy; // remove enemy on refresh
                if(enemy != EN_YOU) {
                    print_str (15, 20, 22, "YOU WIN AND GET", FALSE);
                    sp_UpdateNow ();
                    espera_forzosa (200);
                    print_str (16, 21, 23, "NOTHING", FALSE);
                } else {
                    print_str (19, 20, 22, "I WIN.", FALSE);
                    sp_UpdateNow ();
                    espera_forzosa (200);
                }
                break;
            }
        }

        if(gpjt != (gpyy + (gpxx * 2))) {
            peta_el_beeper (3);
            gpjt = gpyy + (gpxx * 2);
        }else
            espera_forzosa (1);
                    
        combat_update_stats ();
	
	    #asm
		    ; Music generated by beepola
		    call player
	    #endasm

	
    }

    sp_UpdateNow ();
    espera_forzosa (300);

	#asm
	  di
	#endasm

	blackout ();
}

void game_over (void) {

	print_str (10, 12, 79, " GAME OVER ", TRUE);
	sp_UpdateNow ();
		
	gpit = 3;
	peta_el_beeper (10);
	do{
		peta_el_beeper (9);
	}while (--gpit);
	
	espera_activa (500);
}

signed int addsign (signed int n, signed int value) {
    if (!n)
        return 0;
	else if (n >= 0) 
	    return value; 
	else return -value;
}

#if defined(PLAYER_PUSH_BOXES) || !defined(DEACTIVATE_KEYS) || defined(OPEN_DOORS)
void process_tile (unsigned char x0, unsigned char y0, signed char x1, signed char y1) {
    gpjt = qtile (x0, y0);

	if (gpjt == T_ENDKEY) {
		map_attr [15 * y0 + x0] = 0;
		draw_coloured_tile (VIEWPORT_X + (x0<<1), VIEWPORT_Y + (y0<<1), 0);
		player.keys |= F_ENDKEY;
        draw_coloured_tile(22 ,21 , T_ENDKEY);
		peta_el_beeper (8);
    }

    if (gpjt == T_CHAMPI) {
        player.keys |= F_CHAMPI;
		peta_el_beeper (0);
        draw_coloured_tile(20 ,21 , T_CHAMPI);
        draw_scr();
    }
    
    
    if (gpjt == T_PASTI) {
        player.keys |= F_PASTI;
		map_attr [15 * y0 + x0] = 0;
		draw_coloured_tile (VIEWPORT_X + (x0<<1), VIEWPORT_Y + (y0<<1), 0);
        draw_coloured_tile(15 ,21 , T_PASTI);
		peta_el_beeper (3);
    }
    
    if (gpjt == T_MALETA) {
        player.keys |= F_MALETA;
		map_attr [15 * y0 + x0] = 0;
		draw_coloured_tile (VIEWPORT_X + (x0<<1), VIEWPORT_Y + (y0<<1), 0);
        draw_coloured_tile(17 ,21 , T_MALETA);
		peta_el_beeper (3);
    }
    
    

#ifdef OPEN_DOORS
	gpit = (joyfunc) (&keys);
	if ((gpit & sp_FIRE) == 0) {

        if(is_door(gpjt)){
		    // abrir
		    map_attr [15 * y0 + x0] = 0;
		    map_buff [15 * y0 + x0] = 0;
		    // Pintar
		    draw_coloured_tile (VIEWPORT_X + (x0<<1), VIEWPORT_Y + (y0<<1) - 2, 0);
		    draw_coloured_tile (VIEWPORT_X + (x0<<1), VIEWPORT_Y + (y0<<1), 0);
		    // Sonido abrir puerta
		    peta_el_beeper (1);
      	    player.estado = EST_OPEN;
        }
        if((gpjt == 45) && (player.keys & F_ENDKEY) && (player.keys & F_PASTI))
            player.estado = EST_WIN;
	}
#endif
}
#endif

unsigned char move (void) {
	gpit = (joyfunc) (&keys); // Leemos del teclado

	/* Por partes. Primero el movimiento vertical. La ecuacin de movimien-
	   to viene a ser, en cada ciclo:

	   1.- vy = vy + g
	   2.- y = y + vy

	   O sea la velocidad afectada por la gravedad. 
	   Para no colarnos con los nmeros, ponemos limitadores:
	*/

#ifndef PLAYER_MOGGY_STYLE
	// Si el tipo de movimiento no es MOGGY_STYLE, entonces nos afecta la gravedad.
	if (player.vy < PLAYER_MAX_VY_CAYENDO)
		player.vy += PLAYER_G;
	else
		player.vy = PLAYER_MAX_VY_CAYENDO;
#ifdef PLAYER_CUMULATIVE_JUMP
	if (!player.saltando)
#endif
	if (player.gotten) player.vy = 0;		
#else
	// Si lo es, entonces el movimiento vertical se comporta exactamente igual que 
	// el horizontal.
	if ( ! ((gpit & sp_UP) == 0 || (gpit & sp_DOWN) == 0)) {
		player.facing_v = 0xff;
		if (player.vy > 0) {
			player.vy -= PLAYER_RX;
			if (player.vy < 0)
				player.vy = 0;
		} else if (player.vy < 0) {
			player.vy += PLAYER_RX;
			if (player.vy > 0)
				player.vy = 0;
		}
	}

	if ((gpit & sp_UP) == 0) {
		player.facing_v = FACING_UP;
		if (player.vy > -PLAYER_MAX_VX) {
			player.vy -= PLAYER_AX;
		}
	}

	if ((gpit & sp_DOWN) == 0) {
		player.facing_v = FACING_DOWN;
		if (player.vy < PLAYER_MAX_VX) {
			player.vy += PLAYER_AX;
		}
	}
#endif

#ifdef PLAYER_HAS_JETPAC
	if ((gpit & sp_UP) == 0) {
		player.vy -= PLAYER_INCR_JETPAC;
		if (player.vy < -PLAYER_MAX_VY_JETPAC) player.vy = -PLAYER_MAX_VY_JETPAC;
	}
#endif

	player.y += player.vy;
	
	// Safe
		
	if (player.y < 0)
		player.y = 0;
		
	if (player.y > 9216)
		player.y = 9216;

	
	/* El problema es que no es tan fcil... Hay que ver si no nos chocamos.
	   Si esto pasa, hay que "recular" hasta el borde del obstculo.

	   Por eso miramos el signo de vy, para que los clculos sean ms sencillos.
	   De paso vamos a precalcular un par de cosas para que esto vaya ms rpido.
	*/

	gpx = player.x >> 6;				// dividimos entre 64 para pixels, y luego entre 16 para tiles.
	gpy = player.y >> 6;
	gpxx = gpx >> 4;
	gpyy = gpy >> 4;
	
	// Ya	
	possee = 0;
	hit_v = 0;

	if (player.vy < 0) { 			// estamos ascendiendo
		if ((gpy & 15) < 2) {
			if ( ((gpx & 15) < 12 && attr (gpxx, gpyy) & 8) || ((gpx & 15) > 4 && attr (gpxx + 1, gpyy) & 8)) {
				player.vy = 0;
				player.y = ((gpyy + 1) << 10) - 896;
				wall = WTOP;
			}
		}
	} else if (player.vy > 0 && (gpy & 15) < 8) { 	// estamos descendiendo
		if (player.y < 9216) {
			if ( ((gpx & 15) < 12 && attr (gpxx, gpyy + 1) & 12) || ((gpx & 15) > 4 && attr (gpxx + 1, gpyy + 1) & 12)) {
				player.vy = 0;
				player.y = gpyy << 10;
				wall = WBOTTOM;
				possee = 1;
			}
		}
	}

	
	// ------ ok con el movimiento vertical.

	/* Movimiento horizontal:

	   Mientras se pulse una tecla de direccin, 
	   
	   x = x + vx
	   vx = vx + ax

	   Si no se pulsa nada:

	   x = x + vx
	   vx = vx - rx
	*/

	if ( ! ((gpit & sp_LEFT) == 0 || (gpit & sp_RIGHT) == 0)) {
#ifdef PLAYER_MOGGY_STYLE		
		player.facing_h = 0xff;
#endif
		if (player.vx > 0) {
			player.vx -= PLAYER_RX;
			if (player.vx < 0)
				player.vx = 0;
		} else if (player.vx < 0) {
			player.vx += PLAYER_RX;
			if (player.vx > 0)
				player.vx = 0;
		}
	}

	if ((gpit & sp_LEFT) == 0) {
#ifdef PLAYER_MOGGY_STYLE
		player.facing_h = FACING_LEFT;
#endif
		if (player.vx > -PLAYER_MAX_VX) {
#ifndef PLAYER_MOGGY_STYLE			
			player.facing = 0;
#endif
			player.vx -= PLAYER_AX;
		}
	}

	if ((gpit & sp_RIGHT) == 0) {
#ifdef PLAYER_MOGGY_STYLE	
		player.facing_h = FACING_RIGHT;
#endif
		if (player.vx < PLAYER_MAX_VX) {
			player.vx += PLAYER_AX;
#ifndef PLAYER_MOGGY_STYLE						
			player.facing = 1;
#endif
		}
	}

	player.x = player.x + player.vx;
	
	// Safe
	
	if (player.x < 0)
		player.x = 0;
		
	if (player.x > 14336)
		player.x = 14336;
		
	/* Ahora, como antes, vemos si nos chocamos con algo, y en ese caso
	   paramos y reculamos */

	gpy = player.y >> 6;
	gpx = player.x >> 6;
	gpyy = gpy >> 4;
	gpxx = gpx >> 4;
	hit_h = 0;
#ifdef BOUNDING_BOX_8_BOTTOM	
	if (player.vx < 0 && (gpx & 15) < 12) {
		if ( ((gpy & 15) < 2 && attr (gpxx, gpyy) & 8) || ((gpy & 15) && attr (gpxx, gpyy + 1) & 8)) {
			player.vx = 0;
			player.x = ((gpxx + 1) << 10) - 256;
			wall = WLEFT;
		}
	} else if ((gpx & 15) >= 4) {
		if ( ((gpy & 15) < 2 && attr (gpxx + 1, gpyy) & 8) || ((gpy & 15) && attr (gpxx + 1, gpyy + 1) & 8)) {
			player.vx = 0;
			player.x = (gpxx << 10) + 256;
			wall = WRIGHT;
		}
	}
#else
#ifdef BOUNDING_BOX_8_CENTERED
	if (player.vx < 0 && (gpx & 15) < 12) {
		if ( ((gpy & 15) < 12 && attr (gpxx, gpyy) & 8) || ((gpy & 15) > 4 && attr (gpxx, gpyy + 1) & 8)) {
			player.vx = 0;
			player.x = ((gpxx + 1) << 10) - 256;
			wall = WLEFT;
		} else if ( ((gpy & 15) < 8 && attr (gpxx, gpyy) & 1) || ((gpy & 15) && attr (gpxx, gpyy + 1) & 1)) {
			hit_h = 1; 
		}
	} else if ((gpx & 15) >= 4) {
		if ( ((gpy & 15) < 12 && attr (gpxx + 1, gpyy) & 8) || ((gpy & 15) > 4 && attr (gpxx + 1, gpyy + 1) & 8)) {
			player.vx = 0;
			player.x = (gpxx << 10) + 256;
			wall = WRIGHT;
		} else if ( ((gpy & 15) < 8 && attr (gpxx + 1, gpyy) & 1) || ((gpy & 15) && attr (gpxx + 1, gpyy + 1) & 1)) {
			hit_h = 1; 
		}
	}
#endif
#endif

#ifdef PLAYER_MOGGY_STYLE
	// Priority to decide facing
	#ifdef TOP_OVER_SIDE
		if (player.facing_v != 0xff) {
			player.facing = player.facing_v;
		} else if (player.facing_h != 0xff) {
			player.facing = player.facing_h;
		}
	#else
		if (player.facing_h != 0xff) {
			player.facing = player.facing_h;
		} else if (player.facing_v != 0xff) {
			player.facing = player.facing_v;
		}
	#endif	
#endif

#ifdef FIRE_TO_PUSH	
pushed_any = 0;
#endif

#if defined(PLAYER_PUSH_BOXES) || !defined(DEACTIVATE_KEYS) || defined(OPEN_DOORS)
	// Empujar cajas (tile #14)
	gpx = player.x >> 6;
	gpy = player.y >> 6;
	gpxx = gpx >> 4;
	gpyy = gpy >> 4;
	
	// En modo plataformas, no se puede empujar verticalmente
#ifdef PLAYER_MOGGY_STYLE
	
	if (wall == WTOP) {
 #if defined(BOUNDING_BOX_8_BOTTOM) || defined (BOUNDING_BOX_8_CENTERED)
		if (attr (gpxx, gpyy) == 10) {				
			process_tile (gpxx, gpyy, gpxx, gpyy - 1);
 #else
		if (attr (gpxx, gpyy - 1) == 10) {				
			process_tile (gpxx, gpyy - 1, gpxx, gpyy - 2);
 #endif
		}
		if ((gpx & 15)) {
 #if defined(BOUNDING_BOX_8_BOTTOM) || defined (BOUNDING_BOX_8_CENTERED)
			if (attr (gpxx + 1, gpyy) == 10) {
				process_tile (gpxx + 1, gpyy, gpxx + 1, gpyy - 1);
 #else
			if (qtile (gpxx + 1, gpyy - 1) == 10) {
				process_tile (gpxx + 1, gpyy - 1, gpxx + 1, gpyy - 2);
 #endif
			}
		}
	} else if (wall == WBOTTOM) {
		if (attr (gpxx, gpyy + 1) == 10) {
			process_tile (gpxx, gpyy + 1, gpxx, gpyy + 2);
		}
		if ((gpx & 15)) {
			if (attr (gpxx + 1, gpyy + 1) == 10) {
				process_tile (gpxx + 1, gpyy + 1, gpxx + 1, gpyy + 2);
			}	
		}
	}
#endif
	// Horizontalmente
	
	if (wall == WRIGHT) {
		if (attr (gpxx + 1, gpyy) == 10) {
			process_tile (gpxx + 1, gpyy, gpxx + 2, gpyy);
		}
		if ((gpy & 15)) {
			if (attr (gpxx + 1, gpyy + 1) == 10) {
				process_tile (gpxx + 1, gpyy + 1, gpxx + 2, gpyy + 1);
			}
		}
	} else if (wall == WLEFT) {
#if defined(BOUNDING_BOX_8_BOTTOM) || defined(BOUNDING_BOX_8_CENTERED)
		if (attr (gpxx, gpyy) == 10) {
			process_tile (gpxx, gpyy, gpxx - 1, gpyy);
#else
		if (attr (gpxx - 1, gpyy) == 10) {
			process_tile (gpxx - 1, gpyy, gpxx - 2, gpyy);
#endif
		}
		if ((gpy & 15)) {
#if defined(BOUNDING_BOX_8_BOTTOM) || defined(BOUNDING_BOX_8_CENTERED)
			if (attr (gpxx, gpyy + 1) == 10) {
				process_tile (gpxx, gpyy + 1, gpxx - 1, gpyy + 1);
#else
			if (attr (gpxx - 1, gpyy + 1) == 10) {
				process_tile (gpxx - 1, gpyy + 1, gpxx - 2, gpyy + 1);
#endif
			}
		}
	}						
#endif

#ifdef PLAYER_CAN_FIRE
	// Disparos
#ifdef FIRE_TO_PUSH	
	if ((gpit & sp_FIRE) == 0 && player.disparando == 0 && !pushed_any) {
#else
	if ((gpit & sp_FIRE) == 0 && player.disparando == 0) {
#endif
		player.disparando = 1;
		fire_bullet ();
	}
	
	if ((gpit & sp_FIRE)) 
		player.disparando = 0;

#endif

#ifndef DEACTIVATE_EVIL_TILE
	// Tiles que te matan. 
	// hit_v tiene preferencia sobre hit_h
	hit = 0;
	if (hit_v) {
		hit = 1;
#ifdef FULL_BOUNCE
		player.vy = addsign (-player.vy, PLAYER_MAX_VX);
#else
		player.vy = -player.vy;
#endif
	} else if (hit_h) {
		hit = 1;
#ifdef FULL_BOUNCE
		player.vx = addsign (-player.vx, PLAYER_MAX_VX);
#else
		player.vx = -player.vx;
#endif
	}
	if (hit) {
#ifdef PLAYER_FLICKERS
		if (player.life > 0 && player.estado == EST_NORMAL) {
			player.estado = EST_PARP;
			player.ct_estado = 50;
#else
		if (player.life > 0) {
#endif		
			peta_el_beeper (4);
			player.life --;
		}
	}
#endif

	// Select animation frame 
	
#ifndef PLAYER_MOGGY_STYLE
 #ifdef PLAYER_BOOTEE
	gpit = player.facing << 2;
	if (player.vy == 0) {
		player.next_frame = player_frames [gpit];
	} else if (player.vy < 0) {
		player.next_frame = player_frames [gpit + 1];
	} else {
		player.next_frame = player_frames [gpit + 2];
	}
 #else	
	if (!possee && !player.gotten) {
		player.next_frame = player_frames [8 + player.facing];
	} else {
		gpit = player.facing << 2;
		if (player.vx == 0) {
  #ifdef PLAYER_ALTERNATE_ANIMATION
			player.next_frame = player_frames [gpit];
  #else
			player.next_frame = player_frames [gpit + 1];
  #endif
		} else {
			++player.subframe;
			if (player.subframe == 4) {
				player.subframe = 0;
  #ifdef PLAYER_ALTERNATE_ANIMATION
				player.frame ++; if (player.frame == 3) player.frame = 0;
  #else
				player.frame = (player.frame + 1) & 3;
  #endif
  #ifdef PLAYER_STEP_SOUND
				step ();
  #endif
			}
			player.next_frame = player_frames [gpit + player.frame];
		}
	}
 #endif
#else
	
	if (player.vx || player.vy) {
		++player.subframe;
		if (player.subframe == 6) {
			player.subframe = 0;
			player.frame = !player.frame;
#ifdef PLAYER_STEP_SOUND
			step (); 
#endif
		}
    // sky - lo quiero parado cuando suelte cualquier tecla
	} else {
	    player.frame = 0;
	}
	
	player.next_frame = player_frames [player.facing + player.frame];
#endif
}


void __FASTCALL__ draw_scr_background (void) {

#ifndef COMPRESSED_MAPS
 #ifdef UNPACKED_MAP
	map_pointer = mapa + (n_pant * 150);
 #else
	map_pointer = mapa + (n_pant * 75);
 #endif
#else
    descomprimir_map (n_pant);
    map_pointer = 0x5df2;
#endif


    // pasti crazyness
    // 7 +  0  * 8 + 64 * 1
    // 7 + rnd * 8 + 64 = 71 + rnd * 8
    if(player.keys & F_CHAMPI){
        if(n_pant != 20)
            wall = 71 + (((rand() % 3) * 8) + 8);
        else
            wall = 71;
        update_color_tileset(wall);
    }

    // malotes type on first tile
    malotes_t = *map_pointer;
    malotes_t &= ~malotes_k; // malotes killed
    
	gpx = gpy = 0;	

	// Draw 150 tiles
	
	for (gpit = 0; gpit < 150; ++gpit) {	
		// Mapa tipo UNPACKED
		gpd = *map_pointer++;

        if(!gpit)
            gpd = 0;
        else if((gpd == T_ENDKEY)&&(player.keys & F_ENDKEY))
            gpd = 0;
        else if((gpd == T_CHAMPI)&&(player.keys & (F_CHAMPI|F_ENDKEY)))
            gpd = 0;
        else if((gpd == T_PASTI) && (player.keys & F_PASTI))
            gpd = 0;
        else if((gpd == T_MALETA)&&(player.keys & F_MALETA))
            gpd = 0;

		put_tile( gpx, gpy, gpd );
		gpx += 2;
		if (gpx == 30) {
			gpx = 0;
			gpy += 2;
		}
	}
	
	// sky - house code
    house_handler();

    // init malotes per screen	
    malotes[0].wait = 16;
    malotes[1].wait = 64;
    malotes[2].wait = 16;

    malotes[0].x = malotes[1].x = 72;
    malotes[0].y = 112;
    malotes[1].y = 106;
    malotes[2].y = malotes[2].x = 120;

    gpit = 0;

    do{
        malotes[gpit].x += (rand () % 92);
        malotes[gpit].y += (rand () % 12);
    }while(++gpit < 3);
    

    // pastilla random
    if((pastilla_rnd == n_pant) && (!(player.keys & F_PASTI)))
        put_tile (16, 16, 26);

}

void __FASTCALL__ draw_scr (void) {

#ifdef ENABLE_FIRE_ZONE
	f_zone_ac = 0;
#endif	

	draw_scr_background ();
	
// eliminar no hace falta enemy_offset
enoffs = 0;

en_an [1].next_frame = sprite_18_a;
en_an [2].next_frame = sprite_18_a;

// nurse
en_an [0].frame = 0;
en_an [0].count = 3;
en_an [0].base_frame = ((1) - 1) << 1;

// guy
en_an [1].base_frame = 6;
}


void select_joyfunc (void) {

    if(playing == 2){
	    #asm
		    ; Music generated by beepola
		    call continue_player
	    #endasm
    } else {
	    #asm
		    ; Music generated by beepola
		    call musicstart
	    #endasm
    }
	
	gpit = 0;
	s_joy_read = 0; // resets music joy read

	while (!gpit) {
		gpjt = sp_GetKey ();
		switch (gpjt) {
			case '1':
				gpit = 1;
				joyfunc = sp_JoyKeyboard;
				break;
			case '2':
				gpit = 1;
				joyfunc = sp_JoyKempston;
				s_joy_read = 1;
				break;
			case '3':
				gpit = 1;
				joyfunc = sp_JoySinclair1;
				break;
		}			
	}
	#asm
		di
	#endasm
}

void mueve_bicharracos (void) {
	gpx = player.x >> 6;
	gpy = player.y >> 6;

	tocado = 0;
	player.gotten = 0;
	
	//print_number2 (1, 1, malotes_t);
    //print_number2(0,2, pastilla_rnd);

    if(malotes[0].wait)
        --malotes[0].wait;
    
    if (malotes_t & EN_GUY) {
        switch(n_pant){
            case 28:
                gpen_x = malotes [1].x;
                gpen_y = malotes [1].y;
                malotes [1].mx = (signed char) (addsign (((gpx >> 2) << 2) - gpen_x, 2)); // 1, -1 o 0
                malotes [1].x += malotes [1].mx;
                gpen_xx = malotes [1].x >> 4;
                gpen_yy = malotes [1].y >> 4;

                malotes [1].my = (signed char) (addsign (((gpy >> 2) << 2) - gpen_y, 2)); // 1, -1 o 0
                malotes [1].y += malotes [1].my;

            default:
            if (en_an [1].count == 8) {
                // move
                possee = rand() % 32;
                if(possee < 3){
                    if (malotes [1].y < 140)
                        ++malotes [1].y;
                } else if (possee < 6) {
                        if (malotes [1].y > 100)
                            --malotes [1].y;
                }

                en_an [1].count = 0;
                en_an [1].frame = !en_an [1].frame;
                en_an [1].next_frame = enem_frames [en_an [1].base_frame + en_an [1].frame];
            }
        }
        ++en_an [1].count; 

        gpen_cx = malotes [1].x;
        gpen_cy = malotes [1].y;

        if (!tocado && collide (gpx, gpy, gpen_cx, gpen_cy) && player.estado == EST_NORMAL) {
            peta_el_beeper(11);
            combat(EN_GUY);
        }
    } else {
   		en_an [1].next_frame = sprite_18_a;
    }

    if (malotes_t & EN_NUR) {
        if (player.estado == EST_NORMAL) {
            if(!malotes[0].wait){
                gpen_x = malotes [0].x;
                gpen_y = malotes [0].y;
                malotes [0].mx = (signed char) (addsign (((gpx >> 2) << 2) - gpen_x, 1)); // 1, -1 o 0
                malotes [0].x += malotes [0].mx;
                gpen_xx = malotes [0].x >> 4;
                gpen_yy = malotes [0].y >> 4;

                malotes [0].my = (signed char) (addsign (((gpy >> 2) << 2) - gpen_y, 1)); // 1, -1 o 0

                malotes [0].y += malotes [0].my;
            }
        }
	
        gpen_cx = malotes [0].x;
        gpen_cy = malotes [0].y;

        if(malotes [0].my < 0 && !malotes [0].mx)
            en_an [0].base_frame = 4;
        else if(malotes [0].mx < 0)
            en_an [0].base_frame = 2;
        else
            en_an [0].base_frame = 0;

        // sprites with greater count are static
        //if( en_an [gpit].count <= 4 ) {
        // Animate
            if (en_an [0].count == 5) {
                en_an [0].count = 0;
                en_an [0].frame = !en_an [0].frame;
                en_an [0].next_frame = enem_frames [en_an [0].base_frame + en_an [0].frame];
            }
            ++en_an [0].count; 
        if (!tocado && collide (gpx, gpy, gpen_cx, gpen_cy) && player.estado == EST_NORMAL) {
            peta_el_beeper(7);
            combat(EN_NUR);
        }
	    //}
    } else {
   		en_an [0].next_frame = sprite_18_a;
    }
}

